"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = require("fs");
var config_collector_1 = require("./config-collector");
var path_1 = require("path");
var ConfigCollection = /** @class */ (function () {
    function ConfigCollection(opts) {
        this.preferedParseMethod = true;
        if (opts != null) {
            this.preferedParseMethod = opts.preferedParseMethod;
        }
        this.files = [];
    }
    /**
     * The directory will be read and then control
     *  will be passed to this.handleFiles
     * @param path Path to directory
     */
    ConfigCollection.prototype.handleDirectory = function (path, extfilter) {
        path = path || process.cwd();
        return this.handleFiles(fs_1.readdirSync(path), path, extfilter);
    };
    /**
     * Take several files into the collection
     * @param files Path to files
     */
    ConfigCollection.prototype.handleFiles = function (files, root, extfilter) {
        files = files || fs_1.readdirSync(process.cwd());
        var _files = [];
        files.forEach(function (file) {
            if (root != null)
                file = path_1.join(root, file);
            if (extfilter != null && path_1.extname(file) == extfilter) {
                _files.push(new config_collector_1.ConfigCollector(file));
            }
            else if (extfilter == null) {
                _files.push(new config_collector_1.ConfigCollector(file));
            }
        });
        this.files = _files;
        return this;
    };
    /**
     * Fetch files in directory. Doesn't save files to the instance.
     * Instead of config.handleDirectory('./config').files.forEach(file =>
     *  console.log(file.fullPath);
     * );
     * @param dirname
     */
    ConfigCollection.prototype.fetch = function (dirname, cb) {
        if (fs_1.existsSync(dirname)) {
            fs_1.readdirSync(dirname).forEach(function (file) {
                cb(new config_collector_1.ConfigCollector(path_1.join(dirname, file)));
            });
        }
    };
    ConfigCollection.prototype.getFiles = function () {
        if (this.files == null || this.files.length == 0) {
            return [];
        }
        else {
            var res_1 = [];
            this.files.forEach(function (file) {
                if (file.name)
                    res_1.push(file.name);
            });
            return res_1;
        }
    };
    /**
     * Find file and maybe take something from that.
     * @param name File name
     * @param selector JsonQuery.Selector
     */
    ConfigCollection.prototype.find = function (name, selector) {
        if (this.files != null) {
            for (var _i = 0, _a = this.files; _i < _a.length; _i++) {
                var file = _a[_i];
                var fname = (file.name || '') + (file.extension || '');
                if (new RegExp(name.replace(/\*/g, '.*')).test(fname)) {
                    if (selector) {
                        return file.query(selector);
                    }
                    else {
                        return file;
                    }
                }
            }
        }
        return null;
    };
    return ConfigCollection;
}());
exports.ConfigCollection = ConfigCollection;
//# sourceMappingURL=config-collection.js.map