import { readdirSync, existsSync } from 'fs';
import { ConfigCollector } from './config-collector';
import { extname, join } from 'path';

interface Options {
  preferedParseMethod: boolean;
}

export class ConfigCollection {
  protected preferedParseMethod: boolean = true;
  public files: ConfigCollector[];

  constructor(opts?: Options) {
    if (opts != null) {
      this.preferedParseMethod = opts.preferedParseMethod;
    }
    this.files = [];
  }

  /**
   * The directory will be read and then control
   *  will be passed to this.handleFiles
   * @param path Path to directory
   */
  handleDirectory(path?: string, extfilter?: string) {
    path = path || process.cwd();
    return this.handleFiles(readdirSync(path), path, extfilter);
  }

  /**
   * Take several files into the collection
   * @param files Path to files
   */
  handleFiles(files: string[], root?: string, extfilter?: string) {
    files = files || readdirSync(process.cwd());
    let _files: ConfigCollector[] = [];
    files.forEach(file => {
      if (root != null)
        file = join(root, file);
      if (extfilter != null && extname(file) == extfilter) {
        _files.push(new ConfigCollector(file));
      } else if (extfilter == null) {
        _files.push(new ConfigCollector(file));
      }
    });
    this.files = _files;
    return this;
  }

  /**
   * Fetch files in directory. Doesn't save files to the instance.
   * Instead of config.handleDirectory('./config').files.forEach(file =>
   *  console.log(file.fullPath);
   * );
   * @param dirname
   */
  fetch(dirname: string, cb: (file: ConfigCollector) => void) {
    if (existsSync(dirname)) {
      readdirSync(dirname).forEach(file => {
        cb(new ConfigCollector(join(dirname, file)));
      });
    }
  }

  getFiles() {
    if (this.files == null || this.files.length == 0) {
      return [];
    } else {
      let res: string[] = [];
      this.files.forEach(file => {
        if (file.name) res.push(file.name);
      });
      return res;
    }
  }

  /**
   * Find file and maybe take something from that.
   * @param name File name
   * @param selector JsonQuery.Selector
   */
  find(name: string, selector?: string) {
    if (this.files != null) {
      for (const file of this.files) {
        let fname = (file.name || '') + (file.extension || '');
        if (new RegExp(name.replace(/\*/g, '.*')).test(fname)) {
          if (selector) {
            return file.query(selector);
          } else {
            return file;
          }
        }
      }
    }
    return null;
  }
}
