"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = require("fs");
var path_1 = require("path");
var config_parser_1 = require("./config-parser");
// import JsonQuery from "json-query";
var JsonQuery = require('json-query');
var ConfigCollector = /** @class */ (function () {
    /**
     * The default parser is the ConfigParser plain
     * @param path Path to configuration file
     * @param preferedParseMethod Sync | Async (undone)
     */
    function ConfigCollector(path, preferedParseMethod) {
        if (preferedParseMethod === void 0) { preferedParseMethod = true; }
        this.preferedParseMethod = true;
        if (fs_1.existsSync(path)) {
            this.path = path;
            this.parser = new config_parser_1.ConfigParser(path_1.extname(this.path));
            this.preferedParseMethod = preferedParseMethod;
        }
    }
    Object.defineProperty(ConfigCollector.prototype, "path", {
        get: function () {
            return this._path || "";
        },
        set: function (path) {
            this._path = path;
            this.name = path_1.basename(path);
            this.extension = path_1.extname(path);
            this.name = this.name.substring(0, this.name.length - this.extension.length);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ConfigCollector.prototype, "fullName", {
        get: function () {
            return this.name + '' + this.extension;
        },
        enumerable: true,
        configurable: true
    });
    ConfigCollector.prototype.getPath = function () {
        return this.path;
    };
    Object.defineProperty(ConfigCollector.prototype, "fullPath", {
        get: function () {
            return path_1.resolve(this.path);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Test the parser"s parsable extension and
     *  if fine set the parser as default
     * @param parser Parser for the config file
     */
    ConfigCollector.prototype.setParser = function (parser) {
        if (this.path == null)
            throw new Error("Path not set");
        if (parser.extension == path_1.extname(this.path)) {
            this.parser = parser;
            return true;
        }
        return false;
    };
    /**
     * Call method parse of the parser
     */
    ConfigCollector.prototype.parse = function (method) {
        method = method || this.preferedParseMethod;
        if (!this.path)
            throw new Error("Path not set");
        if (!this.parser)
            throw new Error("Parser not set");
        if (method) {
            return this.parser.parseSync(this.path);
        }
        else {
            return this.parser.parse(this.path);
        }
    };
    /**
     * Queries are strings that describe an object or value to
     * pluck out, or manipulate from the context object. The
     * syntax is a little bit CSS, a little bit JS, but pretty powerful
     * @example https://www.npmjs.com/package/json-query
     * @param selector
     * @param parsed The main object to query | default this.parse()
     */
    ConfigCollector.prototype.query = function (selector, parsed) {
        return JsonQuery(selector, {
            data: parsed || this.parse(true)
        });
    };
    ConfigCollector.prototype.QueryJson = function (selector, data) {
        return JsonQuery(selector, {
            data: data
        });
    };
    return ConfigCollector;
}());
exports.ConfigCollector = ConfigCollector;
//# sourceMappingURL=config-collector.js.map