import { existsSync } from "fs";
import { basename, extname, resolve } from "path";
import { ConfigParser } from "./config-parser";
// import JsonQuery from "json-query";
const JsonQuery = require('json-query');


export class ConfigCollector {

    public name?: string;
    public extension?: string;

    private _path?: string;
    protected parser?: ConfigParser;

    protected preferedParseMethod: boolean = true;

    /**
     * The default parser is the ConfigParser plain
     * @param path Path to configuration file
     * @param preferedParseMethod Sync | Async (undone)
     */
    constructor(path: string, preferedParseMethod: boolean = true) {
        if (existsSync(path)) {
            this.path = path;
            this.parser = new ConfigParser(extname(this.path));
            this.preferedParseMethod = preferedParseMethod;
        }
    }

    protected set path(path: string) {
        this._path = path;
        this.name = basename(path);
        this.extension = extname(path);
        this.name = this.name.substring(0, this.name.length - this.extension.length);
    }

    protected get path(): string {
        return this._path || "";
    }

    public get fullName(): string {
        return this.name + '' + this.extension;
    }

    public getPath() {
        return this.path;
    }

    public get fullPath() {
        return resolve(this.path);
    }

    /**
     * Test the parser"s parsable extension and
     *  if fine set the parser as default
     * @param parser Parser for the config file
     */
    setParser(parser: ConfigParser) {
        if (this.path == null)
            throw new Error("Path not set");
        if (parser.extension == extname(this.path)) {
            this.parser = parser;
            return true;
        }
        return false;
    }

    /**
     * Call method parse of the parser
     */
    parse(method?: boolean) {
        method = method || this.preferedParseMethod;
        if (!this.path)
            throw new Error("Path not set");
        if (!this.parser)
            throw new Error("Parser not set");
        if (method) {
            return this.parser.parseSync(this.path);
        } else {
            return this.parser.parse(this.path);
        }
    }

    /**
     * Queries are strings that describe an object or value to
     * pluck out, or manipulate from the context object. The
     * syntax is a little bit CSS, a little bit JS, but pretty powerful
     * @example https://www.npmjs.com/package/json-query
     * @param selector
     * @param parsed The main object to query | default this.parse()
     */
    query(selector: string, parsed?: object) {
        return JsonQuery(selector, {
            data: parsed || this.parse(true)
        });
    }

    QueryJson(selector: string, data: object) {
        return JsonQuery(selector, {
            data: data
        });
    }

}