"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Field = /** @class */ (function () {
    function Field(name, datatype, props) {
        this.name = name;
        this.dtype = datatype;
        this.props = props || {
            unique: false,
            primaryKey: false,
            autoIncrement: false,
            unsigned: false
        };
    }
    Field.prototype.getAll = function () {
        return {
            name: this.name,
            type: this.dtype,
            props: this.props
        };
    };
    return Field;
}());
exports.Field = Field;
var Scheme = /** @class */ (function () {
    function Scheme(fields) {
        this.$fields = fields;
    }
    Scheme.prototype.number = function (name, props) {
        this.$fields.push(new Field(name, 'number', props));
    };
    Scheme.prototype.autoIncrements = function (name) {
        if (name === void 0) { name = 'id'; }
        this.$fields.push(new Field(name, 'number', {
            unique: false,
            primaryKey: true,
            unsigned: true,
            autoIncrement: true
        }));
    };
    Scheme.prototype.string = function (name, props) {
        this.$fields.push(new Field(name, 'string', props));
    };
    Scheme.prototype.json = function (name) {
        this.$fields.push(new Field(name, 'object'));
    };
    Scheme.prototype.boolean = function (name, props) {
        this.$fields.push(new Field(name, 'boolean', props));
    };
    Scheme.prototype.test = function (msg) {
        console.log(msg);
    };
    Scheme.prototype.foreign = function (name) {
        var self = this;
        return {
            references: function (tableName) {
                return {
                    on: function (field) {
                        self.$fields.push(new Field(name, "foreign", {
                            foreign: {
                                table: tableName,
                                field: field
                            }
                        }));
                    }
                };
            }
        };
    };
    Scheme.prototype.timestamps = function () {
        this.$fields.push(new Field('created_at', 'datetime'));
        this.$fields.push(new Field('updated_at', 'datetime:new'));
    };
    return Scheme;
}());
exports.Scheme = Scheme;
//# sourceMappingURL=database-field.js.map