interface SchemeDbTypeOptions {
  unique?: boolean;
  primaryKey?: boolean;
  autoIncrement?: boolean;
  unsigned?: boolean;
  foreign?: {
    table: string;
    field: string;
  };
}

export class Field {
  protected name: string;
  protected dtype: string;
  protected props: SchemeDbTypeOptions;

  constructor(name: string, datatype: string, props?: SchemeDbTypeOptions) {
    this.name = name;
    this.dtype = datatype;
    this.props = props || {
      unique: false,
      primaryKey: false,
      autoIncrement: false,
      unsigned: false
    };
  }

  getAll() {
    return {
      name: this.name,
      type: this.dtype,
      props: this.props
    };
  }
}

export class Scheme {
  protected $fields: Field[];

  constructor(fields: Field[]) {
    this.$fields = fields;
  }

  number(name: string, props?: SchemeDbTypeOptions) {
    this.$fields.push(new Field(name, 'number', props));
  }

  autoIncrements(name: string = 'id') {
    this.$fields.push(
      new Field(name, 'number', {
        unique: false,
        primaryKey: true,
        unsigned: true,
        autoIncrement: true
      })
    );
  }

  string(name: string, props?: SchemeDbTypeOptions) {
    this.$fields.push(new Field(name, 'string', props));
  }

  json(name: string) {
    this.$fields.push(new Field(name, 'object'));
  }

  boolean(name: string, props?: SchemeDbTypeOptions) {
    this.$fields.push(new Field(name, 'boolean', props));
  }

  test(msg: any) {
    console.log(msg);
  }

  foreign(name: string) {
    const self = this;
    return {
      references(tableName: string) {
        return {
          on(field: string) {
            self.$fields.push(
              new Field(name, `foreign`, {
                foreign: {
                  table: tableName,
                  field: field
                }
              })
            );
          }
        };
      }
    };
  }

  timestamps() {
    this.$fields.push(new Field('created_at', 'datetime'));
    this.$fields.push(new Field('updated_at', 'datetime:new'));
  }
}
