"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * This is the most important part of addons writing.
 *  All addons must inherit from this class to be accepted
 *
 * Lifeloop
 * install-> (configure->[onMount->onUnmount]->destroy)
 */
var Provider = /** @class */ (function () {
    function Provider() {
    }
    /**
     * Before you can use the provider it's
     *  necessary to install it into the Arcadia. In the
     *  method you create configuration files without access
     *  to whole the application. So it's clear installation
     */
    Provider.prototype.install = function () { };
    /**
     * When the provider is installed. It needs to be configured
     *  probably with access to the application
     */
    Provider.prototype.configure = function () { };
    /**
     * Some providers may need to be mounted for a long usage
     *  then it's necessary to mount the provider to not take
     *  lots of time on starting again and again
     */
    Provider.prototype.onMount = function () { };
    /**
     * Unmount is the first phase of Arcadia shutting down
     */
    Provider.prototype.onUnmount = function () { };
    /**
     * The method is like Migration.down. This will remove
     *  the provider from the project. It may leave some files
     *  to rebuild when it will be installed again
     */
    Provider.prototype.destory = function () { };
    return Provider;
}());
exports.Provider = Provider;
//# sourceMappingURL=provider.js.map